/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands {
  import com.adobe.cairngorm.commands.ICommand;
  import com.adobe.cairngorm.control.CairngormEvent;

  import mx.collections.ArrayCollection;
  import mx.collections.ListCollectionView;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
  import org.ow2.orchestra.designer.bpmn.model.PoolModel;
  import org.ow2.orchestra.designer.bpmn.model.ProcessModel;
  import org.ow2.orchestra.designer.events.ProcessEvent;

  public class DeleteSelectedProcessElementsCMD implements ICommand {
    public function DeleteSelectedProcessElementsCMD() {
    }

    public function execute(event:CairngormEvent):void {
      var processEvt:ProcessEvent = event as ProcessEvent;
      // save process model
      processEvt.processEditorModel.saveModelForUndo();

      var listSelectedElements:ArrayCollection = processEvt.currentlySelectedElements;

      for each (var elementToDelete:Object in listSelectedElements) {
        deleteElementAndDependencies(elementToDelete as AbstractElement, processEvt.processEditorModel.processModel);
      }

      // Clean list of selected elements
      processEvt.processEditorModel.selectedElements.removeAll();

    }


    private function deleteElementAndDependencies(elementToDelete:AbstractElement, processModel:ProcessModel):void {
      var listElementsProcess:ListCollectionView = processModel.elements;
      // remove item
      var itemIndex:int = listElementsProcess.getItemIndex(elementToDelete);
      //May be already deleted (sequence flow with related gateways included in selection for instance)
      if (itemIndex >= 0) {
        listElementsProcess.removeItemAt(itemIndex);
        // If pool model, move all elements.
        if (elementToDelete is PoolModel) {
          if (processModel.y > (elementToDelete as PoolModel).y) {
            processModel.y -= PoolModel.POOL_HEIGHT;
          }
          for each (var o:Object in listElementsProcess) {
            if (o is AbstractElementWithPosition) {
              var aewp:AbstractElementWithPosition = o as AbstractElementWithPosition;
              if (aewp.y > (elementToDelete as PoolModel).y) {
                aewp.y -= PoolModel.POOL_HEIGHT;
              }
            }
          }
        }
      }
    }
  }
}
