/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.commands
{
  import com.adobe.cairngorm.business.ServiceLocator;
  import com.adobe.cairngorm.control.CairngormEvent;

  import mx.controls.Alert;
  import mx.rpc.AsyncToken;
  import mx.rpc.Responder;
  import mx.rpc.events.ResultEvent;
  import mx.rpc.remoting.RemoteObject;

  import org.ow2.orchestra.designer.events.ProcessEditorEvent;
  import org.ow2.orchestra.designer.model.DesignerModelLocator;
  import org.ow2.orchestra.designer.model.OrchestraConnectionPropertiesModel;
  import org.ow2.orchestra.designer.model.ProcessEditorModel;
  import org.ow2.orchestra.designer.vo.FileNavigatorProcess;

  public class ModelDeployCMD extends ModelExportCMD
  {
    public function ModelDeployCMD()
    {
      super();
    }

    public override function execute(event:CairngormEvent):void {
      if (event is ProcessEditorEvent) {
        var pee:ProcessEditorEvent = event as ProcessEditorEvent;
        if (pee.type == ProcessEditorEvent.DEPLOY_MODELS) {
          processEditorModelList = pee.processEditorModelList;
        }
      }
      super.execute(event);
    }

    protected override function exportModelResult(event:ResultEvent = null):void {
      this.deployProcessEditor(processEditorModelList.getItemAt(currentIndex) as ProcessEditorModel);
    }

    private function deployProcessEditor(processEditor:ProcessEditorModel):void {
      var process:FileNavigatorProcess = processEditor.fileNavigatorProcess;
      var remote:RemoteObject = ServiceLocator.getInstance().getRemoteObject("deployer");

      var connectionProperties:OrchestraConnectionPropertiesModel = DesignerModelLocator.getInstance().orchestraConnectionProperties;
      var token:AsyncToken = remote.deployToOrchestra(process, connectionProperties.jmxServiceURL, connectionProperties.jmxObjectName);
      token.addResponder(new Responder(deployModelResult, fault));
    }

    private function deployModelResult(event:ResultEvent):void {
      Alert.show("Process " + event.result + " deployed to orchestra", "Process deployed to orchestra");
      super.exportModelResult();
    }

    protected override function get commandName():String {
      return "deploy";
    }
  }
}