/**
 * Created by ${PRODUCT_NAME}.
 * User: camillej
 * Date: 24/02/11
 * Time: 13:59
 * To change this template use File | Settings | File Templates.
 */
package org.ow2.orchestra.designer.view.processEditor {
  import flash.display.DisplayObject;
  import flash.events.MouseEvent;
  import flash.geom.Point;

  import mx.collections.ArrayCollection;
  import mx.managers.CursorManager;
  import mx.styles.StyleManager;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
  import org.ow2.orchestra.designer.bpmn.model.GatewayModel;
  import org.ow2.orchestra.designer.bpmn.model.PoolModel;
  import org.ow2.orchestra.designer.bpmn.model.SequenceFlowModel;
  import org.ow2.orchestra.designer.bpmn.model.TaskModel;
  import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
  import org.ow2.orchestra.designer.bpmn.views.ProcessPoolView;

  public class ProcessEditorFlowStateHelper {

    protected var processEditorView:ProcessEditorView;

    private static var copyCursor:Class = StyleManager.getStyleDeclaration('DragManager').getStyle('copyCursor');
    private static var rejectedCursor:Class = StyleManager.getStyleDeclaration('DragManager').getStyle('rejectCursor');

    public function ProcessEditorFlowStateHelper(processEditorView:ProcessEditorView) {
      this.processEditorView = processEditorView;
    }

    /**
     * Restore default cursor.
     */
    protected function resetCursor():void {
      CursorManager.removeCursor(CursorManager.currentCursorID);
    }

    protected function allowFlowTarget(mouseEvent:MouseEvent):Boolean {
      var listSelectedElements:ArrayCollection = new ArrayCollection();
      listSelectedElements = getElementUnderMouse(mouseEvent);

      if (listSelectedElements.length != 1) {
        return false;
      }
      // only one selected element.
      var selectedElement:AbstractElementWithPosition = listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
      if (selectedElement is GatewayModel) {
        return true;
      }
      var elementSource:AbstractElement =
              processEditorView.processEditorModel.selectedElements.getItemAt(0) as AbstractElement;
      var sequenceFlowModel:SequenceFlowModel = null;
      if (elementSource != null && elementSource is SequenceFlowModel) {
        sequenceFlowModel = elementSource as SequenceFlowModel;
        elementSource = sequenceFlowModel.sourceElement;
      }
      if (selectedElement is PoolModel) {
        if (sequenceFlowModel != null) {
          // cannot move an existing sequenceFlow to a pool
          return false;
        }
        return elementSource is TaskModel;
      } else {
        if (elementSource == selectedElement) {
          return false;
        }
        if (sequenceFlowModel != null) {
          // case of moving an existing sequence flow
          return selectedElement.incomingSequenceFlows.length < 1 || selectedElement == sequenceFlowModel.targetElement;
        } else if (elementSource is TaskModel) {
          // case of creation a new sequence flow
          // for task model check there is not already a sequence flow
          return (elementSource as TaskModel).outgoingSequenceFlows.length < 1;
        }
        return selectedElement.incomingSequenceFlows.length < 1;
      }
    }

    protected function allowFlowSource(mouseEvent:MouseEvent):Boolean {
      var listSelectedElements:ArrayCollection = new ArrayCollection();
      listSelectedElements = getElementUnderMouse(mouseEvent);

      if (listSelectedElements.length != 1) {
        return false;
      }
      // only one selected element.
      var selectedElement:AbstractElementWithPosition = listSelectedElements.getItemAt(0) as AbstractElementWithPosition;
      if (selectedElement is GatewayModel) {
        return true;
      }
      if (selectedElement is PoolModel) {
        return false;
      } else {
        var sequenceFlowModel:SequenceFlowModel = processEditorView.processEditorModel.selectedElements.getItemAt(0) as SequenceFlowModel;
        if (sequenceFlowModel != null) {
          return selectedElement.outgoingSequenceFlows.length < 1 || selectedElement == sequenceFlowModel.sourceElement;
        }
        return selectedElement.outgoingSequenceFlows.length < 1;
      }
    }

    /**
     * Update mouse cursor with copy or rejected cursor if the drop is accepted/rejected
     * @param acceptDrop is drop allowed ?
     */
    protected function updateDropMouseCursor(acceptDrop:Boolean):void {
      // Delete cursor used during drag operations
      resetCursor();

      if (acceptDrop) {
        // change cursor with drop ok
        CursorManager.setCursor(copyCursor);
      } else {
        CursorManager.setCursor(rejectedCursor);
      }
    }

    /**
     * Gets the list of AbstractElementWithPosition elements view currently under the mouse pointer.
     * @param mouseEvent
     * @return
     */
    protected function getElementUnderMouse(mouseEvent:MouseEvent):ArrayCollection {
      var listSelectedElements:ArrayCollection = new ArrayCollection();
      if (mouseEvent.target != null) {
        // Get mouse event position. The coordinates are relative to the target element
        var targetLocalMousePoint:Point = new Point(mouseEvent.localX, mouseEvent.localY);
        // Transform the coordinates to be relative to this view
        var mousePoint:Point = processEditorView.globalToLocal(DisplayObject(mouseEvent.target).localToGlobal(targetLocalMousePoint));

        // check if mouse is on a element view or on the canvas.
        // find list of elements that are contained under the mouse cursor
        for each (var object:DisplayObject in processEditorView.processView.getChildren()) {
          if (!(object is ProcessPoolView) && object is AbstractElementView && AbstractElementView(object).model is AbstractElementWithPosition) {
            // get the child object display zone (with coordinates relative to this view)
            if (AbstractElementView(object).containsPoint(mousePoint, processEditorView)) {
              // the process element view is contained in the selection area,
              // the attached model should be selected.
              listSelectedElements.addItem((object as AbstractElementView).model);
            }
          }
        }
      }
      return listSelectedElements;
    }
  }
}
