/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.processEditor {
import flash.display.DisplayObject;
import flash.display.InteractiveObject;
import flash.events.Event;
import flash.events.KeyboardEvent;
import flash.events.MouseEvent;
import flash.geom.Point;
import flash.geom.Rectangle;
import flash.ui.Keyboard;

import mx.binding.utils.ChangeWatcher;
import mx.containers.Canvas;
import mx.controls.Image;
import mx.core.Application;
import mx.core.EventPriority;
import mx.events.CloseEvent;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;
import mx.events.DragEvent;
import mx.managers.DragManager;
import mx.managers.IFocusManagerComponent;
import mx.managers.PopUpManager;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.model.EventModel;
import org.ow2.orchestra.designer.bpmn.model.GatewayModel;
import org.ow2.orchestra.designer.bpmn.model.PoolModel;
import org.ow2.orchestra.designer.bpmn.model.TaskModel;
import org.ow2.orchestra.designer.bpmn.model.operations.InterfaceModel;
import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
import org.ow2.orchestra.designer.bpmn.views.IFlowNodeView;
import org.ow2.orchestra.designer.bpmn.views.ProcessPoolView;
import org.ow2.orchestra.designer.bpmn.views.ProcessView;
import org.ow2.orchestra.designer.bpmn.views.utils.Arrow;
import org.ow2.orchestra.designer.events.ProcessEvent;
import org.ow2.orchestra.designer.events.ProcessModificationEvent;
import org.ow2.orchestra.designer.model.ElementPaletteDnDModel;
import org.ow2.orchestra.designer.model.ProcessEditorModel;
import org.ow2.orchestra.designer.utils.DesignerUtil;
import org.ow2.orchestra.designer.view.ConfirmPopup;
import org.ow2.orchestra.designer.view.ElementContextPanel;
import org.ow2.orchestra.designer.view.palette.AbstractElementPalette;
import org.ow2.orchestra.designer.view.palette.EventPaletteView;
import org.ow2.orchestra.designer.view.palette.GatewayPaletteView;
import org.ow2.orchestra.designer.view.palette.TaskPaletteView;
import org.ow2.orchestra.designer.vo.WsdlService;

/**
 *
 * @author hallouin-s
 *
 * This class implements @see(IFocusManagerComponent) to gets the focus
 * on click. This will be used to intercept some events like Keybord events.
 */
public class ProcessEditorView extends Canvas implements IFocusManagerComponent {

  /**
   * The source for "palette" image data binding.
   */
  [Embed(source="/../assets/icon/container/bpmn.gif")]
  public static const processIcon:Class;

  [Bindable]
  private var _processEditorModel:ProcessEditorModel;

  private var processSelectionViewer:ProcessEditorSelectionViewer;

  private var deleteConfirmWindow:ConfirmPopup;

  internal var processView:ProcessView;

  internal var elementContextPanel:ElementContextPanel = new ElementContextPanel();

  /**
   * selection rectangle object.
   * In the "selection" state, defines the selected area.
   * In the "moving" state, defines the previous mouse position (used to update elements positions).
   */
  internal var selectionRect:Canvas;

  /**
   * Variable used for creatingFlow and creatingElement states.
   */
  internal var arrow:Arrow;
  public var imageOfElementToAdd:Image = new Image();
  public var modelOfElementToAdd:AbstractElementWithPosition;

  /** process view should accept dropping of the current object */
  private var acceptDrop:Boolean;
  /** pre-validation of drag is correct */
  private var dragEnterValid:Boolean;

  /**
   *
   * @param processEditorModel
   */
  public function ProcessEditorView(processEditorModel:ProcessEditorModel) {
    _processEditorModel = processEditorModel;
    onProcessModelChange();

    ChangeWatcher.watch(processEditorModel, "processModel", onProcessModelChange);

    this.icon = processIcon;

    this.setStyle("borderStyle", "solid");
    this.setStyle("backgroundColor", "white");

    // add drag enter and drop listeners for objects dragged from the palette
    this.addEventListener(DragEvent.DRAG_ENTER, processViewDragEnter, false, EventPriority.DEFAULT, true);
    this.addEventListener(DragEvent.DRAG_DROP, processViewDrop, false, EventPriority.DEFAULT, true);
    this.addEventListener(DragEvent.DRAG_OVER, processViewDragOver, false, EventPriority.DEFAULT, true);

    this.addEventListener(KeyboardEvent.KEY_DOWN, processEditorViewKeyDown, false, EventPriority.DEFAULT, true);

    this.addEventListener(MouseEvent.MOUSE_OUT, handleMouseOut, false, EventPriority.DEFAULT, true);

    _processEditorModel.selectedElements.addEventListener(CollectionEvent.COLLECTION_CHANGE,
            modelSelectedElementsChanged, false, EventPriority.DEFAULT, true);

    this.addChild(elementContextPanel);
    elementContextPanel.id = "elementContextPanel";
    hideElementContextPanel();

    // create selection rectangle object
    selectionRect = new Canvas();
    selectionRect.setStyle("backgroundColor", "green");
    selectionRect.alpha = 0.5;

    processSelectionViewer = new ProcessEditorSelectionViewer(this);

    //create arrow
    arrow = new Arrow(new Point(0, 0), new Point(0, 0));

    // generate states
    new ProcessEditorDefaultStateHelper(this).addDefaultState();
    new ProcessEditorSelectingStateHelper(this).addSelectingState();
    new ProcessEditorMovingStateHelper(this).addMovingState();
    new ProcessEditorCreatingFlowStateHelper(this).addCreatingFlowState();
    new ProcessEditorCreatingElementStateHelper(this).addCreatingElementState();
    var processEditorMovingFlowStateHelper:ProcessEditorMovingFlowStateHelper = new ProcessEditorMovingFlowStateHelper(this);
    processEditorMovingFlowStateHelper.addMovingSourceState();
    processEditorMovingFlowStateHelper.addMovingTargetState();
    // go to default state
    this.currentState = "default";

    this.automationName =
            "processEditorView-" + processEditorModel.fileNavigatorProcess.project.name + ">"
                    + processEditorModel.fileNavigatorProcess.name;
  }

  private function onProcessModelChange(event:Event = null):void {
    var index:int = 0;
    if (processView != null) {
      index = this.getChildIndex(processView);
      this.removeChildAt(index);
    }
    processView = new ProcessView(_processEditorModel.processModel);
    processView.percentHeight = 100;
    processView.percentWidth = 100;
    this.addChildAt(processView, index);

    this.label = _processEditorModel.processModel.name;
  }

  /**
   * Handle keyBoardEvent
   * @param event
   */
  private function processEditorViewKeyDown(event:KeyboardEvent):void {
    var numberElement:int = _processEditorModel.selectedElements.length;

    if (event.keyCode == Keyboard.DELETE && numberElement > 0) {
      // remove temporary keydown listener
      this.removeEventListener(KeyboardEvent.KEY_DOWN, processEditorViewKeyDown);

      // create the windows to confirm deletion
      deleteConfirmWindow = ConfirmPopup(PopUpManager.createPopUp(Application.application as Application, ConfirmPopup, true));
      deleteConfirmWindow.title = "Warning : BPMN elements is to be deleted...";
      deleteConfirmWindow.addEventListener(CloseEvent.CLOSE, itemDeleteHandler);
      deleteConfirmWindow.addEventListener(ConfirmPopup.CANCEL, itemDeleteHandler);

      if (numberElement == 1) {
        deleteConfirmWindow.file.text = "Do you want to delete this element ?\rThis operation can't be undone.";
      }
      else {
        deleteConfirmWindow.file.text = "Do you want to delete this " + numberElement + " elements ?\rThis operation can't be undone.";
      }

      PopUpManager.centerPopUp(deleteConfirmWindow);
      deleteConfirmWindow.setFocus();
    }
  }

  private function itemDeleteHandler(event:CloseEvent):void {
    if (event.type == CloseEvent.CLOSE) {
      new ProcessEvent(
              ProcessEvent.PROCESS_SELECTED_ELEMENTS_DELETE,
              _processEditorModel).dispatch();
    }

    //add keydown listener temporaly deleted during deleteConfirmWindow modal windows opening
    this.addEventListener(KeyboardEvent.KEY_DOWN, processEditorViewKeyDown);

  }

  /**
   *
   * @param event
   */
  private function modelSelectedElementsChanged(event:Event):void {
    // event instance of CollectionEvent
    // check kind attribute for "add, delete, move..."
    // get added or removed child
    if (event is CollectionEvent) {
      var collectionEvent:CollectionEvent = event as CollectionEvent;
      // model elements changed.
      var changedElements:Array = collectionEvent.items;
      switch (collectionEvent.kind) {
        case CollectionEventKind.ADD:
          for each (var addedElement:Object in changedElements) {
            updateViewElement(addedElement, true);
          }
          // show context panel if a new element is selected.
          if (this._processEditorModel.selectedElements.length == 1) {
            showElementContextPanel();
          }
          break;

        case CollectionEventKind.REMOVE:
          for each (var removedElement:Object in changedElements) {
            updateViewElement(removedElement, false);
          }
          break;

        case CollectionEventKind.RESET:
          for each (var element:Object in processView.processModel.elements) {
            updateViewElement(element, false);
          }
          break;
      }
    }
    // show context panel if possible
    showElementContextPanel();
  }


  /**
   * Hide the context panel for a process element
   */
  internal function hideElementContextPanel():void {
    this.elementContextPanel.setVisible(false);
  }

  /**
   * Show the context panel for a process element:
   * set the panel attached view to the current selected object and show the panel.
   */
  internal function showElementContextPanel():void {
    if (this.currentState == "default" && this._processEditorModel.selectedElements.length == 1 &&
            this._processEditorModel.selectedElements.getItemAt(0) is AbstractElement) {
      // get current element
      var selectedElement:AbstractElement = this._processEditorModel.selectedElements.getItemAt(0) as AbstractElement;
      if (selectedElement != null) {
        // get associated view
        var selectedElementView:AbstractElementView = this.processView.getElementViewByModel(selectedElement);
        // set context panel attached view (updates the panel content and position)
        elementContextPanel.attachedView = selectedElementView;
        // reset context panel state
        elementContextPanel.currentState = "default";
        // show panel
        elementContextPanel.setVisible(true);
      }
    } else {
      hideElementContextPanel();
    }
  }

  /**
   * Called when an object is selected/unselected.
   */
  private function updateViewElement(element:Object, isToBeSelected:Boolean):void {
    processSelectionViewer.updateViewElement(element, isToBeSelected);
  }

  /**
   *
   * @return
   */
  public function get processEditorModel():ProcessEditorModel {
    return _processEditorModel;
  }

  /** Object drag handler */
  private function processViewDragEnter(event:DragEvent):void {
    this.dragEnterValid = false;
    this.acceptDrop = false;
    // Accept object
    if (event.dragInitiator is AbstractElementPalette) {
      DragManager.acceptDragDrop(this);
      DragManager.showFeedback(DragManager.COPY);
      this.acceptDrop = true;
      this.dragEnterValid = true;
    } else if (event.dragSource.hasFormat("items")) {
      var items:Array = event.dragSource.dataForFormat("items") as Array;
      if (items.length == 1) {
        var item:Object = items[0];
        if (item is WsdlService) {
          DragManager.acceptDragDrop(this);
          DragManager.showFeedback(DragManager.COPY);
          this.acceptDrop = true;
          this.dragEnterValid = false;
        }
      }
    }
  }

  /** Object drag handler */
  private function processViewDragOver(event:DragEvent):void {
    if (dragEnterValid) {
      if (event.dragInitiator is AbstractElementPalette) {
        var elementRectangle:Rectangle;
        var elemDnDModel:ElementPaletteDnDModel = event.dragSource.dataForFormat("paletteItem") as ElementPaletteDnDModel;
        elementRectangle = elemDnDModel.image.getRect(this);
        if (elementRectangle.topLeft.x < ProcessPoolView.POOL_HEADER_WIDTH
                || elementRectangle.topLeft.y < processView.processModel.y) {
          DragManager.showFeedback(DragManager.NONE);
          acceptDrop = false;
        } else {
          DragManager.showFeedback(DragManager.COPY);
          acceptDrop = true;
          //Check collisions
          for each (var elementView:AbstractElementView in processView.getChildren()) {
            if ((elementView is IFlowNodeView) &&
                    IFlowNodeView(elementView).getCollisionRectangle(this).intersects(elementRectangle)) {
              DragManager.showFeedback(DragManager.NONE);
              acceptDrop = false;
              return;
            }
          }
        }
      } else {
        // Accept object
        if (event.localX < ProcessPoolView.POOL_HEADER_WIDTH
                || event.localY < processView.processModel.y) {
          DragManager.showFeedback(DragManager.NONE);
          acceptDrop = false;
        } else {
          DragManager.showFeedback(DragManager.COPY);
          acceptDrop = true;
        }
      }
    }
  }

  /**
   * Object drop handler.
   * This method is called when an object is droped in the process view
   */
  private function processViewDrop(event:DragEvent):void {
    if (acceptDrop) {
      var dataModel:AbstractElementWithPosition = null;
      // get the dropped object
      if (event.dragInitiator is GatewayPaletteView) {
        dataModel = new GatewayModel();
      } else if (event.dragInitiator is EventPaletteView) {
        dataModel = new EventModel();
        dataModel.name = "event";
      } else if (event.dragInitiator is TaskPaletteView) {
        dataModel = new TaskModel();
        dataModel.name = "task";
      } else if (event.dragSource.hasFormat("items")) {
        // Test if drag from the WSDL navigator
        var items:Array = event.dragSource.dataForFormat("items") as Array;
        if (items.length == 1) {
          var item:Object = items[0];
          if (item is WsdlService) {
            // Create a new pool
            dataModel = new PoolModel();
            dataModel.name = (item as WsdlService).name;

            var iface:InterfaceModel = DesignerUtil.copy((item as WsdlService).interfaceModel) as InterfaceModel;
            iface.poolModel = dataModel as PoolModel;
            (dataModel as PoolModel).interfaces.put(iface.name, iface);
            (dataModel as PoolModel).wsdlNamespace = (item as WsdlService).namespace;
            (dataModel as PoolModel).wsdlPath = (item as WsdlService).wsdlPath;
          }
        }
      }

      var targetX:int = event.localX;
      var targetY:int = event.localY;
      if (event.dragSource.hasFormat("paletteItem")) {
        var elemDnDModel:ElementPaletteDnDModel = event.dragSource.dataForFormat("paletteItem") as ElementPaletteDnDModel;
        targetX -= elemDnDModel.deltaX;
        targetY -= elemDnDModel.deltaY;
      }


      // if a new element is added, generate a process modification event.
      if (dataModel != null) {
        new ProcessModificationEvent(ProcessModificationEvent.PROCESS_ELEMENT_ADD,
                this.processEditorModel, dataModel,
                targetX, targetY).dispatch();
      }
    }
  }

  /**
   * handler for mouse out event.
   * when the mouse leaves this view, the view goes back to the "default" state.
   */
  private function handleMouseOut(event:MouseEvent):void {
    // First check if we leave this view.
    // Mouse out events can be generated when the mouse goes over a child view of this view.

    // set target to the new object under the mouse.
    var target:InteractiveObject = event.relatedObject;
    // look recursively for a process element view
    // return when first element is found.
    while (target != null) {
      if (target is AbstractElementView || target == this) {
        // new object is a child of this view, return.
        return;
      }
      if (target == Application.application) {
        // we leave this view, go back to the "default" state.
        this.currentState = "default";
        return;
      } else if (target is DisplayObject) {
        // look in the parent
        target = (target as DisplayObject).parent;
      }
    }
  }

}
}
