/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.view.properties {
  import flash.events.Event;
  import flash.events.FocusEvent;

  import mx.controls.Label;
  import mx.controls.TextInput;
  import mx.core.UIComponent;
  import mx.events.FlexEvent;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.events.ElementPropertyChangeEvent;

  [Bindable]
public class TextProperty extends AbstractProperty {
  private var _value:String;
  private var _itemEditor:TextInput;
  private var _itemRenderer:Label;


  public function TextProperty(propertyName:String, description:String, value:String, isEditable:Boolean = false) {
    super(propertyName, description);
    _value = value;
    _itemRenderer = new Label();
    _itemRenderer.text = value;
    if (isEditable) {
      _itemEditor = new TextInput();
      _itemEditor.text = value;
      _itemEditor.automationName = propertyName + "Editor";

      _itemEditor.addEventListener(FlexEvent.ENTER, this.valueChanged, false, 0, true);
      _itemEditor.addEventListener(FocusEvent.FOCUS_OUT, this.valueChanged, false, 0, true);
    }
  }

  public override function get itemEditor():UIComponent {
    return _itemEditor;
  }


  public override function get itemRenderer():UIComponent {
    return _itemRenderer;
  }

  private function valueChanged(event:Event):void {
    if (_value != _itemEditor.text) {
      var element:AbstractElement = getSelectedElement();
      if (element == null) {
        return;
      }
      _value = _itemEditor.text;
      new ElementPropertyChangeEvent(ElementPropertyChangeEvent.PROPERTY_CHANGED, element, _propertyName,
              _itemEditor.text).dispatch();
    }
  }
}
}
